
#include <windows.h>
#include <process.h>
#include <string>
#include <sstream>
#include <regex>
#include <iostream>
#include <iomanip>
#include <fstream>
using namespace std;
#pragma warning(disable: 4244) // '=' : conversion from 'double' to 'float', possible loss of data

#define verify(expression) { SetLastError(0); if (!(expression)) verify_failed(#expression, __FILE__, __LINE__); }
void verify_failed(const char* expression, const char* file, int line) {
  DWORD e = GetLastError();
  string s = string(expression) + " failed in " + file + " line " + to_string(line);
  if (e) {
    char msg[2048];
    if (FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM, NULL, e, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), msg, sizeof(msg), NULL)) {
      s += string("\n\n") + msg;
    }
  }
  throw s;
}

void SetClipboardData(const void* data, size_t size, UINT format) {
  verify(OpenClipboard(NULL));
  verify(EmptyClipboard());
  HGLOBAL hmem;
  verify(hmem = GlobalAlloc(GMEM_MOVEABLE, size));
  void* ptr;
  verify(ptr = GlobalLock(hmem));
  memcpy(ptr, data, size);
  verify(GlobalUnlock(hmem) == 0 && GetLastError() == 0);
  verify(SetClipboardData(format, hmem));
  verify(CloseClipboard());
}

#include "haptix.h"
#define MJ(mj_function) { if ((mj_function) != mjCOM_OK) mj_function_failed(#mj_function, __FILE__, __LINE__); }
void mj_function_failed(char* mj_function, char* file, int line) {
  throw string(mj_function) + " failed in " + file + " line " + to_string(line) + "\n\n" + hx_last_result();
}

HWND Haptix_window;
BOOL CALLBACK Find_haptix_window(HWND hwnd, LPARAM) {
  char title[20];
  if (GetWindowTextA(hwnd, title, sizeof(title)) && strncmp(title, "MuJoCo HAPTIX", 13) == 0) {
    Haptix_window = hwnd;
    return false;
  }
  return true;
}
unsigned _stdcall Watchdog(void*) {
  while (IsWindow(Haptix_window)) Sleep(200);
  exit(1);
  return 0;
}

int APIENTRY wWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow) {
  HANDLE watchdog = INVALID_HANDLE_VALUE;
  try {
    EnumWindows(Find_haptix_window, 0);
    if (Haptix_window) {
      SwitchToThisWindow(Haptix_window, true);
#ifdef _DEBUG
    } else if (WinExec("\"H:\\mjhaptix131\\program\\mjhaptix.exe\" -nomocap", SW_SHOWMAXIMIZED) >= 32) {
      EnumWindows(Find_haptix_window, 0);
      verify(Haptix_window);
      PostMessage(Haptix_window, WM_COMMAND, 1000, 0);
      //int wait = 3000 / 100;
      //while (!GetWindow(Haptix_window, GW_CHILD) && wait--) Sleep(100);
      //HWND canvas;
      //verify(canvas = GetWindow(Haptix_window, GW_CHILD));
      //PostMessage(canvas, WM_KEYDOWN, VK_SPACE, 0);
      //PostMessage(canvas, WM_KEYDOWN, VK_F4, 0);
#endif
    } else throw string("Cannot find MuJoCo HAPTIX window");
    verify((watchdog = (HANDLE)_beginthreadex(NULL, 4096, Watchdog, NULL, STACK_SIZE_PARAM_IS_A_RESERVATION, NULL)) != INVALID_HANDLE_VALUE);
    MJ(mj_connect(NULL));
    ostringstream ss;
    ss << "<!--" << endl;
    int num = 0;
    mjOneBody ob = {0};
    mjtResult result;
    while ((result = mj_get_onebody(&ob)) == mjCOM_OK) {
      if (ob.isfloating) {
        const char* s;
        MJ((s = mj_id2name("body", ob.bodyid)) ? mjCOM_OK : -1);
        ss << setw(16) << (*s ? s : string("body ") + to_string(ob.bodyid)) << ':';
        ss << " pos=\"" << ob.pos[0] << ' ' << ob.pos[1] << ' ' << ob.pos[2] << '"';
        ss << " quat=\"" << ob.quat[0] << ' ' << ob.quat[1] << ' ' << ob.quat[2] << ' ' << ob.quat[3] << '"';
        ss << endl;
        num++;
      }
      ob.bodyid++;
    }
    if (result != mjCOM_BADINDEX) throw string("mj_get_onebody(&ob) failed\n\n") + hx_last_result();
    ss << "-->" << endl;
    TerminateThread(watchdog, 0);
    SetClipboardData(ss.str().c_str(), ss.str().length() + 1, CF_TEXT);
    MessageBoxA(Haptix_window, (string("Positions and quats of ") + to_string(num) + " floating bodies have been copied to the clipboard.").c_str(), "Get Floating Body Positions", MB_OK);
    return 0;
  } catch (string& s) {
    TerminateThread(watchdog, 0);
    Sleep(100);
    if (IsWindow(Haptix_window) || mj_result() != mjCOM_CANNOTRECV) {
      MessageBoxA(Haptix_window, s.c_str(), "Nuts and Bolts Tool", MB_ICONERROR | MB_OK);
    }
    return 0;
  }
}
