
render_R = false; // 3 minutes
render_H = false; // 1.5 hours
segment = -1; // [0 : 36] or -1 for everything

W = [13.1, 10.1]; // wrench widths for both ends in mm
W_avg = (W[0] * 2 + W[1]) / 3; // compromise between both ends
H = W_avg * .13; // main height
R = .25; // reinforcement height * W[e]
L = W_avg * 4; // halflength handle
rh = .4; // halfwidth handle * W[e]
$fa = 8; $fs = 0.8; // circle smoothness
offset_segments = 0; // oversizing the segments by .01 mm should prevent drawing errors in OpenGL but it doesn't

module end_2d(W) {
  scale(W / 140) for (m = [0, 1]) mirror([0, m]) translate([-258, -233]) polygon([
    [231,233], [232,217], [235,202], [240,190], [246,180],
    [252,173], [259,167], [265,164], [272,163], [384,163],
    [386,161], [386,159], [381,154], [371,145], [356,133],
    [337,120], [314,108], [292, 98], [268, 91], [254, 90],
    [241, 91], [223, 95], [207,103], [194,111], [179,126],
    [167,142], [159,161], [152,182], [149,199], [147,215],
    [146,228], [145,233]
  ]);
}

module reinforcement_2d(W) {
  scale(W / 140) offset(16) for (m = [0, 1]) mirror([0, m]) translate([-258, -233]) polygon([
    [231,233], [232,217], [235,202], [240,190], [246,180],
    [252,173], [259,167], [265,164], [272,163], [295,163],
    [312,164], [332,167], [352,171], [400,184], [400,233]
  ]);
}

module end_with_reinforcement(W, H, R) {
  intersection() {
    minkowski() {
      linear_extrude(1e-5, center = true, convexity = 4) reinforcement_2d(W);
      if (render_R) rotate_extrude($fn = 12, convexity = 4) scale((R - H) / 2) for (m = [0, 1]) mirror([0, m]) polygon([[0,0], [0,1], [.13,.97], [.24,.9], [.71,.3], [.97,.14], [1.3,.05], [2,0]]);
    }
    linear_extrude(R, center = true, convexity = 4) offset(-R / 14) end_2d(W);
  }
  linear_extrude(1e-5, center = true, convexity = 4) offset(-R / 14) end_2d(W);
}

module inlay_2d() {
  hull() for (e = [0, 1]) rotate(e * 180) {
    y = rh * W[e];
    translate([-L - y * 1.113, 0]) circle(y * .28, $fn = 12);
    intersection() {
      translate([-L,  y * 1.8]) circle(y * 2.4);
      translate([-L, -y * 1.8]) circle(y * 2.4);
      translate([-L, 0]) square(y * 2.51, center = true);
    }
  }
}

module sphere_for_minkowski(r = 1, $fn = 16) {
  rotate_extrude() intersection() {
    translate([0, -r]) square([r, 2 * r]);
    circle(r);
  }
}

intersection() {
  union() {
    difference() {
      minkowski() {
        difference() {
          union() {
            for (e = [0, 1]) translate([(e ? W[1] : -W[0]) * 1.4 + (e ? L : -L), 0]) rotate(e ? 15 : 180 + 15) end_with_reinforcement(W[e], H, R * W[e]);
            ru = 1.40; // radius upper circle * W[e]
            rl = 1.05; // radius lower circle * W[e]
            linear_extrude(1e-5, center = true, convexity = 4) difference() {
              polygon([
                [-L - W[0] * .765, (-rh - .33) * W[0]],
                [-L - W[0] * .98, (rh + .4) * W[0]],
                [L + W[1] * .765, (rh + .33) * W[1]],
                [L + W[1] * .98, (-rh - .4) * W[1]]
              ]);
              hull() {
                translate([-L, (rh + ru) * W[0]]) circle(ru * W[0]);
                translate([L, (rh + rl) * W[1]]) circle(rl * W[1]);
              }
              hull() {
                translate([-L, -(rh + rl) * W[0]]) circle(rl * W[0]);
                translate([L, -(rh + ru) * W[1]]) circle(ru * W[1]);
              }
            }
          }
          linear_extrude(1e-5, center = true) offset(render_H ? H * .25 : 0) inlay_2d();
        }
        if (render_H) scale([.5, .5, 1]) sphere_for_minkowski(H * .5, $fn = 20);
      }
      if (render_H) for (e = [0, 1]) translate([(e ? W[1] : -W[0]) * 1.4 + (e ? L : -L), 0]) rotate(e ? 15 : 180 + 15) linear_extrude(R * W[e], center = true) difference() {
        translate([-W[e] / 2, -W[e] / 2]) square([W[e] * 2, W[e]]);
        end_2d(W[e]);
      }
    }
    for (m = [0, 1]) mirror([0, 0, m]) difference() {
      linear_extrude(H * .33) offset(H * .125) inlay_2d();
      translate([0, 0, H * .375]) minkowski() {
        linear_extrude(H) offset(-H * .053) inlay_2d();
        sphere_for_minkowski(H * .125, $fn = 12);
      }
    }
    for (e = [0, 1]) translate([e ? L : -L, 0, H * .25]) linear_extrude(H * .25) rotate(e ? -90 : 90) text(str(floor(W[e])), size = W[e] * .25, halign = "center", valign = "top");
  }
  if (segment >= 0 && segment < 34) translate([0, 0, segment == 32 || segment == 33 ? H * 2.25 : 0]) linear_extrude(H * 4, center = true) offset(offset_segments) {
    s = floor(segment / 2);
    e = segment % 2;
    w = e ? -W[1] : W[0];
    x = (e ? L : -L) - w * 1.4;
    p = [
      /* 0*/ [x + w * -.98, .425 * w], [x + w * -.6, .603 * w], [x + w * -.288, .7 * w], [x + w * -.2, .33 * w], [x + w * -1.2, .1 * w],
      /* 5*/ [x + w * -.085, .7 * w],
      /* 6*/ [x + w * .125, .6 * w], [x + w * -.12, .3 * w],
      /* 8*/ [x + w * .33, .36 * w], [x + w * -.03, .17 * w],
      /*10*/ [x + w * .43, .12 * w], [x + w * .03, .01 * w],
      /*12*/ [x + w * .465, -.14 * w], [x + w * .05, -.165 * w],
      /*14*/ [x + w * .415, -.435 * w], [x + w * .03, -.305 * w],
      /*16*/ [x + w * .26, -.66 * w], [x + w * -.01, -.38 * w],
      /*18*/ [x + w * .105, -.745 * w],
      /*19*/ [x + w * -.21, -.82 * w], [x + w * -.71, -.855 * w], [x + w * -1., -.69 * w],
      /*22*/ [x + w * .915, .995 * w], [x + w * -.5, 1.82 * w],
      /*24*/ [x + w * .915, .09 * w], [x + w * .915, -.09 * w], [x + w * .915, -.8 * w],
      /*27*/ [x + w * .2, -1.6 * w],
      /*28*/ [e ? L : -L, .550 * w], [e ? L : -L, .225 * w],
      /*30*/ [e ? L : -L, -.550 * w], [e ? L : -L, -.225 * w],
      /*32*/ [x + w * 1.36, -.225 * w], [x + w * 1.36, .225 * w], [x + w * 1.68, .225 * w], [x + w * 1.68, -.225 * w]
    ];
    if (s == 0) polygon(p, paths = [[0,1,2,3,4]]);
    else if (s == 1) polygon(p, paths = [[2,3,5]]);
    else if (s == 2) polygon(p, paths = [[3,5,6,7]]);
    else if (s == 3) polygon(p, paths = [[7,6,8,9]]);
    else if (s == 4) polygon(p, paths = [[9,8,10,11]]);
    else if (s == 5) polygon(p, paths = [[11,10,12,13]]);
    else if (s == 6) polygon(p, paths = [[13,12,14,15]]);
    else if (s == 7) polygon(p, paths = [[15,14,16,17]]);
    else if (s == 8) polygon(p, paths = [[17,16,18]]);
    else if (s == 9) polygon(p, paths = [[17,18,19,20,21]]);
    else if (s == 10) polygon(p, paths = [[0,1,2,5,6,22,23]]);
    else if (s == 11) polygon(p, paths = [[12,10,8,6,22,24,25]]);
    else if (s == 12) polygon(p, paths = [[12,14,16,26,25]]);
    else if (s == 13) polygon(p, paths = [[26,16,18,19,20,27]]);
    else if (s == 14) polygon(p, paths = [[24,22,28,29]]);
    else if (s == 15) polygon(p, paths = [[25,26,30,31]]);
    else if (s == 16) polygon(p, paths = [[32,33,34,35]]);
  }
  else if (segment == 34) linear_extrude(H * 3, center = true) offset(offset_segments) polygon([[-L, .225 * W[0]], [-L, .550 * W[0]], [L, .550 * W[1]], [L, .225 * W[1]]]);
  else if (segment == 35) linear_extrude(H * 3, center = true) offset(offset_segments) polygon([[-L, -.225 * W[0]], [-L, -.550 * W[0]], [L, -.550 * W[1]], [L, -.225 * W[1]]]);
  else if (segment == 36) {
    x = [-L - W[0] * 1.4 + W[0] * .915, L + W[1] * 1.4 - W[1] * .915];
    linear_extrude(H * .5 - 1e-6, center = true) offset(offset_segments) polygon([[-L, -.225 * W[0]], [x[0], -.09 * W[0]], [x[0], .09 * W[0]], [-L, .225 * W[0]], [L, .225 * W[1]], [x[1], .09 * W[1]], [x[1], -.09 * W[1]], [L, -.225 * W[1]]]);
  }
}
