
name = "M8 thread segment "; // file name prefix
segment = 0; // [0 : 2] for export as stl
D = 8; // diameter in mm
P = 1.25; // pitch in mm
L = 4 + 6.5 + 1.6; // length in mm +/- P/4
pos_m = [0, 0, 0]; // position in meters
$fn = 60; // facets number
tab = "&nbsp; &nbsp; "; // xml result for MuJoCo in Console window
// after copying & pasting, search & replace ECHO: " by an empty string

module puts(s) {
  echo(str("<b>", s, "</b>ECHO: "));
}

num_segs = round(2 * L / P + 3 * P / 2);
if (num_segs < 4) {
  echo("ERROR: L too small!");
} else {
  puts(str("&lt;asset&gt;"));
  for (s = [0 : 2]) puts(str(tab, "&lt;mesh file=\"", name, s, ".stl\" scale=\".001 .001 .001\" /&gt;"));
  puts(str("&lt;/asset&gt;"));
  puts(str("&lt;body name=\"bolt\" childclass=\"steel\" pos=\"0 0 0\"&gt;"));
  puts(str(tab, "&lt;joint type=\"free\" /&gt;"));
  for (s = [0 : num_segs - 1]) {
    ex = min(max(0, s - num_segs + 3), 1);
    ez = s % 2;
    puts(str(
      tab, "&lt;geom class=\"nb thread\"",
      " mesh=\"", name, [min(s, 2), num_segs - s - 1][ex], "\"",
      " pos=\"", pos_m[0], " ", pos_m[1], " ", pos_m[2] + (s - ex) * P / 2 * 0.001, "\"",
      " euler=\"", ex * 180, " 0 ", ez * 180, "\"",
      " /&gt;"
    ));
  }
  puts(str(tab, "&lt;!--hex head z = ", (num_segs - 2) * P / 2, "--&gt;"));
  puts(str("&lt;/body&gt;"));
}

intersection() {
  difference() {
    H = cos(30) * P;
    R_maj = D / 2;
    R_SVi = R_maj - H * 7 / 8; // Sharp-V inner
    for (s = [0 : $fn / 2]) {
      a0 = 180 - (s - 0.5) * 360 / $fn;
      a1 = 180 - (s + 0.0) * 360 / $fn;
      a2 = 180 - (s + 0.5) * 360 / $fn;
      a3 = 180 - (s + 1.0) * 360 / $fn;
      x0 = R_SVi * sin(a0);
      y0 = R_SVi * cos(a0);
      z0 = (s - 0.5) * P / $fn - P;
      x1 = R_SVi * sin(a2);
      y1 = R_SVi * cos(a2);
      z1 = (s + 0.5) * P / $fn - P;
      x2 = R_maj * sin(a1);
      y2 = R_maj * cos(a1);
      z2 = P / 2 - P / 16 + (s + 0.0) * P / $fn - P;
      x3 = R_maj * sin(a3);
      y3 = R_maj * cos(a3);
      z3 = P / 2 - P / 16 + (s + 1.0) * P / $fn - P;
      polyhedron(
        points = [
          [x0, y0, z0], [x1, y1, z1],
          [x2, y2, z2], [x3, y3, z3],
          [x2, y2, z2 + P / 8], [x3, y3, z3 + P / 8],
          [x0, y0, z0 + P], [x1, y1, z1 + P],
          [0, 0, P / 4 - P], [0, 0, P / 4]
        ],
        faces = [
          [0,1,2], [3,2,1],
          [2,3,4], [5,4,3],
          [4,5,7], [7,6,4],
          [8,1,0],
          [9,6,7],
          [8,0,2], [8,2,4], [9,8,4], [9,4,6],
          [8,3,1], [8,5,3], [9,5,8], [9,7,5]
        ]
      );
    }
    translate([-D, -D, -P * 2]) cube([D, D * 2, P * 3.5]);
    translate([-D, -D, -P * (1 + segment / 2)]) cube([D * 2, D * 2, P]);
  }
  h = D * sin(30) / cos(30);
  translate([0, 0, -h / 2 + h / 8 - P * segment / 2 - P / 16]) {
    cylinder(r1 = 0, r2 = D, h = h);
    translate([0, 0, h]) cylinder(r = D, h = h);
  }
}
