
D = 8; // diameter in mm
P = 1.25; // pitch in mm
Zn = P * 5; // thickness in mm
$fn = 60; // thread facets number

H = cos(30) * P;
R_maj = D / 2;
R_SVi = R_maj - H * 7 / 8; // Sharp-V inner from bolt

s = 360 / $fn;
function snap(a, o) = (round(a / s) + o) * s;

// pressure balls
if (1) {
  c = P * .15 + (R_maj + R_SVi) / 2;
  r = P * .26;
  n = 3; // each upper and lower side
  for (u = [0 : 1]) for (i = [0 : n - 1]) {
    al = 180 + i / n * 360;
    zl = P * al / 360;
    a = snap(u ? Zn % P / P * 360 - al : al, 0);
    x = c * -sin(a);
    y = c * cos(a);
    z = u ? Zn - zl : zl;
    translate([x, y, z]) color("skyblue") sphere(r);
  }
}

// friction balls
if (1) {
  al = 240;
  au = (Zn / P + 1) * 360 - al;
  r = P * .1;
  c = R_maj + r * .8;
  n = 4;
  for (i = [0 : n - 1]) {
    a = snap(al + (au - al) * i / (n - 1), i / n);
    x = c * -sin(a);
    y = c * cos(a);
    z = a * P / 360 - P / 2;
    translate([x, y, z]) color("skyblue") sphere(r);
  }
}

R_SVo = D / 2 + H / 8; // Sharp-V outer radius
R_min = D / 2 - H * 5 / 8;
for (s = [-$fn : Zn / P * $fn]) {
  a0 = 180 - (s + 0.0) * 360 / $fn;
  a1 = 180 - (s + 0.5) * 360 / $fn;
  a2 = 180 - (s + 1.0) * 360 / $fn;
  a3 = 180 - (s + 1.5) * 360 / $fn;
  x0 = R_SVo * sin(a0);
  y0 = R_SVo * cos(a0);
  z0 = (s + 0.0) * P / $fn;
  x1 = R_SVo * sin(a2);
  y1 = R_SVo * cos(a2);
  z1 = (s + 1.0) * P / $fn;
  x2 = R_min * sin(a1);
  y2 = R_min * cos(a1);
  z2 = P / 2 - P / 8 + (s + 0.5) * P / $fn;
  x3 = R_min * sin(a3);
  y3 = R_min * cos(a3);
  z3 = P / 2 - P / 8 + (s + 1.5) * P / $fn;
  polyhedron(
    points = [
      [x0, y0, z0], [x1, y1, z1],
      [x2, y2, z2], [x3, y3, z3],
      [x2, y2, z2 + P / 4], [x3, y3, z3 + P / 4],
      [x0, y0, z0 + P], [x1, y1, z1 + P]
    ],
    faces = [
      [0,2,1], [3,1,2],
      [2,4,3], [5,3,4],
      [5,4,7], [6,7,4],
    ]
  );
}
// inner chamfer
h = D * sin(30) / cos(30);
#translate([0, 0, Zn - h / 2]) cylinder(r1 = 0, r2 = D, h = h);
#translate([0, 0, 0 - h / 2]) cylinder(r1 = D, r2 = 0, h = h);
