
class = "M8 "; // for MuJoCo commands in console window
D = 8; // diameter in mm
P = 1.25; // pitch in mm
Waf = 13; // width across flats in mm (wrench size)
Zn = 6.5; // thickness in mm
pos_m = [0,0,0]; // position in meters
cr = P * .32; // contact ball radius
cvo = P * .2; // contact-visual offset
fn_thread = 60; // facets number of thread for snapping in
$fn = 36; // facets number of nut, visual only
geom_names = true; // for debugging collisions
tab = "&nbsp; &nbsp; &nbsp; "; // xml result for MuJoCo in Console window
// after copying & pasting, search & replace ECHO: " by an empty string

s = 360 / fn_thread;
function snap(a, o) = (round(a / s) + o) * s;

H = cos(30) * P;
R_maj = D / 2;
R_SVi = R_maj - H * 7 / 8; // Sharp-V inner from bolt

module puts(s) {
  echo(str("<b>", s, "</b>ECHO: "));
}

puts(str("&lt;asset&gt;"));
puts(str(tab, "&lt;mesh file='visible ", class, "nut.stl' scale='.001 .001 .001' /&gt;"));
puts(str("&lt;/asset&gt;"));
puts(str("&lt;body name='", class, "nut' childclass='steel' pos='0 0 0'&gt;"));
puts(str(tab, "&lt;joint type='free' /&gt;"));
puts(str(tab, "&lt;geom type='mesh' mesh='visible ", class, "nut' group='2' mass='1e-9' contype='0' conaffinity='0' pos='", pos_m[0], " ", pos_m[1], " ", pos_m[2], "' /&gt;"));

if (1) {
  puts(str(tab, "&lt;!--side boxes for centering thread and collisions with other objects--&gt;"));
  play_sides = D * .001;
  hwx = Waf / 2 * sin(30) / cos(30);
  hwy = (Waf - D - play_sides) / 4;
  rc = Waf / 2 - hwy;
  for (i = [0 : 5]) {
    a = i * 60;
    puts(str(tab, "&lt;geom class='", class, "side' ",
      geom_names ? str("name='side ", i, "' ") : "",
      "size='", hwx * .001 - D * 1e-5, " ", hwy * .001, " ", (Zn / 2 - cvo) * .001, "' pos='",
      pos_m[0] + rc * sin(a) * .001, " ",
      pos_m[1] + rc * cos(a) * .001, " ",
      pos_m[2] + Zn / 2 * .001, "' euler='0 0 ", -a, "'/>"
    ));
  }
}

if (1) {
  puts(str(tab, "&lt;!--friction balls tangential on thread in loose mode--&gt;"));
  al = 240;
  au = (Zn / P + 1) * 360 - al;
  r = P * .1;
  c = R_maj + r * .8;
  n = 4;
  for (i = [0 : n - 1]) {
    a = snap(al + (au - al) * i / (n - 1), i / n);
    x = c * -sin(a);
    y = c * cos(a);
    z = a * P / 360 - P / 2;
    puts(str(tab, "&lt;geom ",
      "class='", class, "friction' ",
      geom_names ? str("name='friction ", i, "' ") : "",
      "size='", r * .001, "' ",
      "pos='", pos_m[0] + x * .001, " ", pos_m[1] + y * .001, " ", pos_m[2] + z * .001, "' ",
      "/&gt;"
    ));
  }
}

if (1) {
  puts(str(tab, "&lt;!--pressure balls for axial clamp force on thread--&gt;"));
  c = P * .15 + (R_maj + R_SVi) / 2;
  r = P * .26;
  n = 3; // each upper and lower side
  for (u = [0 : 1]) for (i = [0 : n - 1]) {
    al = 180 + i / n * 360;
    zl = P * al / 360;
    a0 = snap(u ? Zn % P / P * 360 - al : al, i / n);
    for (a = [a0 - s/4, a0, a0 + s/4]) {
      x = c * -sin(a);
      y = c * cos(a);
      z = u ? Zn - zl : zl;
      puts(str(tab, "&lt;geom ",
        "class='", class, "pressure' ",
        //geom_names ? str("name='pressure ", u * n + i, "' ") : "",
        "size='", r * .001, "' ",
        "pos='", pos_m[0] + x * .001, " ", pos_m[1] + y * .001, " ", pos_m[2] + z * .001, "' ",
        "/&gt;"
      ));
    }
  }
}

if (1) {
  puts(str(tab, "&lt;!--contact balls for axial clamp force on workpiece--&gt;"));
  x = Waf / 2 / cos(30) - cr * sqrt(1 + pow(tan(30), 2)) + D * .01;
  zl = cr - cvo;
  for (u = [0 : 1]) for (i = [0 : 2]) {
    z = u ? Zn - zl : zl;
    a = i * 120 - 30 + u * 60;
    puts(str(tab, "&lt;geom ",
      "class='", class, "contact' ",
      geom_names ? str("name='contact ", u * 3 + i, "' ") : "",
      "size='", cr * .001, "' ",
      "pos='", x * sin(a) * .001, " ", x * cos(a) * .001, " ", z * .001, "' ",
      "/&gt;"
    ));
  }
}

puts(str("&lt;/body&gt;"));

intersection() {
  difference() {
    R_cvo = D / 2 + H / 8; // Sharp-V outer radius
    R_min = D / 2 - H * 5 / 8;
    union() {
      for (s = [-$fn : Zn / P * $fn]) {
        a0 = 180 - (s + 0.0) * 360 / $fn;
        a1 = 180 - (s + 0.5) * 360 / $fn;
        a2 = 180 - (s + 1.0) * 360 / $fn;
        a3 = 180 - (s + 1.5) * 360 / $fn;
        x0 = R_cvo * sin(a0);
        y0 = R_cvo * cos(a0);
        z0 = (s + 0.0) * P / $fn;
        x1 = R_cvo * sin(a2);
        y1 = R_cvo * cos(a2);
        z1 = (s + 1.0) * P / $fn;
        x2 = R_min * sin(a1);
        y2 = R_min * cos(a1);
        z2 = P / 2 - P / 8 + (s + 0.5) * P / $fn;
        x3 = R_min * sin(a3);
        y3 = R_min * cos(a3);
        z3 = P / 2 - P / 8 + (s + 1.5) * P / $fn;
        polyhedron(
          points = [
            [x0, y0, z0], [x1, y1, z1],
            [x2, y2, z2], [x3, y3, z3],
            [x2, y2, z2 + P / 4], [x3, y3, z3 + P / 4],
            [x0, y0, z0 + P], [x1, y1, z1 + P]
          ],
          faces = [
            [0,2,1], [3,1,2],
            [2,4,3], [5,3,4],
            [5,4,7], [6,7,4],
            [0,6,4,2],
            [1,3,5,7],
            [0,1,7,6]
          ]
        );
      }
      difference() {
        cylinder(d = Waf / cos(30), h = Zn, $fn = 6);
        cylinder(r = R_cvo, h = Zn);
      }
    }
    translate([0, 0, Zn + P]) cube([D * 2, D * 2, P * 2], center=true);
    translate([0, 0, 0 - P]) cube([D * 2, D * 2, P * 2], center=true);
    // inner chamfer
    h = D * sin(30) / cos(30);
    translate([0, 0, Zn - h / 2]) cylinder(r1 = 0, r2 = D, h = h);
    translate([0, 0, 0 - h / 2]) cylinder(r1 = D, r2 = 0, h = h);
  }
  // outer chamfer
  h = Waf * sin(30) / cos(30);
  translate([0, 0, 0 - h / 2]) {
    translate([0, 0, Zn]) cylinder(r1 = Waf, r2 = 0, h = h);
    cylinder(r = Waf, h = Zn);
  }
  translate([0, 0, 0 - h / 2]) {
    cylinder(r1 = 0, r2 = Waf, h = h);
    translate([0, 0, h]) cylinder(r = Waf, h = Zn);
  }
}
