
void verify_failed(const char* expression, const char* file, int line) {
  DWORD e = GetLastError();
  string s = string(expression) + " failed in " + file + " line " + to_string(line);
  if (e) {
    char msg[2048];
    if (FormatMessageA(FORMAT_MESSAGE_FROM_SYSTEM, NULL, e, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), msg, sizeof(msg), NULL)) {
      s += string("\n\n") + msg;
    }
  }
  throw s;
}

void mj_function_failed(char* mj_function, char* file, int line) {
  throw string(mj_function) + " failed in " + file + " line " + to_string(line) + "\n\n" + hx_last_result();
}

size_t towcs(const char* ansi, wchar_t* buffer, size_t countof_buffer) {
  size_t wcslen;
  verify((wcslen = MultiByteToWideChar(CP_UTF8, 0, ansi, -1, buffer, (int)countof_buffer)) > 0);
  verify(wcslen <= countof_buffer);
  verify(buffer[--wcslen] == 0);
  return wcslen;
}

const wchar_t* GetProgramDirectory(const char* filename_to_add) {
  static wchar_t buffer[MAX_PATH];
  static wchar_t* name_within_buffer = NULL;
  if (!name_within_buffer) {
    DWORD i = GetModuleFileNameW(NULL, buffer, _countof(buffer));
    verify(i < _countof(buffer));
    verify(i > 3);
    if (buffer[1] == ':') buffer[0] = toupper(buffer[0]);
    while (i > 0 && buffer[i - 1] != '\\') {
      i--;
    }
    name_within_buffer = buffer + i;
  }
  towcs(filename_to_add, name_within_buffer, buffer + sizeof(buffer) - name_within_buffer);
  return buffer;
}

HWND WinWaitActive(const char* substr_title, int timeout) {
  for (;;) {
    HWND hwnd = GetForegroundWindow();
    char title[1024];
    verify(GetWindowTextA(hwnd, title, sizeof(title)));
    if (strstr(title, substr_title)) return hwnd;
    if (timeout <= 0) return NULL;
    int ms = rbound(timeout, 100);
    Sleep(ms);
    timeout -= ms;
  }
}

void SetClipboardData(const void* data, size_t size, UINT format) {
  verify(OpenClipboard(NULL));
  verify(EmptyClipboard());
  HGLOBAL hmem;
  verify(hmem = GlobalAlloc(GMEM_MOVEABLE, size));
  void* ptr;
  verify(ptr = GlobalLock(hmem));
  memcpy(ptr, data, size);
  verify(GlobalUnlock(hmem) == 0 && GetLastError() == 0);
  verify(SetClipboardData(format, hmem));
  verify(CloseClipboard());
}
inline void SetClipboardText(const char* str) { SetClipboardData(str, strlen(str) + 1, CF_TEXT); }
inline void SetClipboardTextW(const wchar_t* wcs) { SetClipboardData(wcs, wcslen(wcs) * 2 + 2, CF_UNICODETEXT); }

const UINT ALT = MOD_ALT << 16;
const UINT CTRL = MOD_CONTROL << 16;
const UINT SHIFT = MOD_SHIFT << 16;
const UINT WIN = MOD_WIN << 16;

void Send(UINT key) {
  static INPUT ip[10];
  int n = 0;
  if (key & ALT) ip[n++].ki.wVk = VK_MENU;
  if (key & CTRL) ip[n++].ki.wVk = VK_CONTROL;
  if (key & SHIFT) ip[n++].ki.wVk = VK_SHIFT;
  if (key & ALT) ip[n++].ki.wVk = VK_LWIN;
  ip[n++].ki.wVk = LOWORD(key);
  for_(i, n) {
    ip[i].type = INPUT_KEYBOARD;
    ip[i].ki.dwFlags = 0;
    ip[n + i].ki.wVk = ip[n - 1 - i].ki.wVk;
    ip[n + i].type = INPUT_KEYBOARD;
    ip[n + i].ki.dwFlags = KEYEVENTF_KEYUP;
  }
  verify(SendInput(n * 2, ip, sizeof(INPUT)));
}

struct TABLE {

  struct COLUMN {
    string header1, header2;
    vector<string> values;
    size_t width;
    int group = 0;
    COLUMN(const char* h1, const char* h2) : header1(h1), header2(h2), width(max(header1.length(), header2.length())) {}
    void push_back(const char* value, size_t row) {
      while (values.size() < row) values.push_back(string());
      values.push_back(value);
      setmax(width, values.back().length());
    }
    bool operator<(const COLUMN& other) const {
      return group < other.group;
    }
  };

  vector<COLUMN> columns;
  int group = 0;
  size_t num_rows = 0;
  bool align_columns = true;
  string separator = "  ";
  char horizontal_line_char = '=';

  COLUMN& find_column(const char* h1, const char* h2) {
    for (COLUMN& c : columns) {
      if (c.header1 == h1 && c.header2 == h2) return c;
    }
    columns.emplace_back(h1, h2);
    return columns.back();
  }

  void write(const char* h1, const char* h2, const char* value, int group = 0) {
    COLUMN& c = find_column(h1, h2);
    c.push_back(value, num_rows);
    c.group = group;
  }

  void align_rows() {
    for (COLUMN& c : columns) setmax(num_rows, c.values.size());
  }

  void print() {
    save(cout);
  }

  void save_headers(ostream& ofs, int h) {
    for (size_t ci = 0; ci < columns.size(); ci++) {
      COLUMN& c = columns[ci];
      if (ci) ofs << separator;
      if (align_columns) ofs << setw(c.width);
      ofs << (h == 1 ? c.header1 : c.header2);
    }
    ofs << endl;
  }

  void save(ostream& ofs) {
    align_rows();
    stable_sort(columns.begin(), columns.end());
    save_headers(ofs, 1);
    size_t has_headers2 = 0;
    for (COLUMN& c : columns) has_headers2 |= c.header2.length();
    if (has_headers2) save_headers(ofs, 2);
    if (horizontal_line_char) {
      size_t w = (columns.size() - 1) * separator.length();
      for (COLUMN& c : columns) w += c.width;
      while (w--) ofs << horizontal_line_char;
      ofs << endl;
    }
    for (size_t r = 0; r < num_rows; r++) {
      for (size_t ci = 0; ci < columns.size(); ci++) {
        COLUMN& c = columns[ci];
        if (ci) ofs << separator;
        if (align_columns) ofs << setw(c.width);
        if (r < c.values.size()) ofs << c.values[r];
        else ofs << "";
      }
      ofs << endl;
    }
  }

  void save(const char* filename) {
    ofstream ofs(filename);
    if (!ofs.good()) throw string("Can't create file ") + filename;
    save(ofs);
  }

  void clear() {
    columns.clear();
    num_rows = 0;
  }

};


#ifdef _DEBUG
_declspec(thread) bool Global_thread_log_state = true;
_declspec(thread) int Global_thread_log_group = 0;
TABLE Log_table;
ofstream Log_file;
struct _loghere {
  bool saved_state;
  int saved_group;
  _loghere(bool state, int group = 0) {
    saved_state = Global_thread_log_state, Global_thread_log_state = state;
    saved_group = Global_thread_log_group, Global_thread_log_group = group;
  }
  operator bool() { 
    return Global_thread_log_state; 
  }
  ~_loghere() { 
    Global_thread_log_state = saved_state;
    Global_thread_log_group = saved_group;
  }
};
void _log(bool force_output = false) {
  if (Global_thread_log_state) {
    if (!Log_file.is_open()) Log_file.open("_DEBUG Nuts and Bolts Tool for HAPTIX.log");
    verify(Log_file.good());
    TABLE& t = Log_table;
    t.align_rows();
    if (t.num_rows >= 45 || t.num_rows > 0 && force_output) {
      Log_file << endl;
      t.save(Log_file);
      t.clear();
    }
  }
}
void _cdecl _log(const char* format, ...) {
  if (Global_thread_log_state) {
    char buffer[1024];
    int n = vsnprintf_s(buffer, sizeof(buffer), _TRUNCATE, format, (char*)(&format + 1));
    if (n < 0) n = sizeof(buffer) - 1;
    if (!strchr(format, '=')) {
      _log(true);
      Log_file << buffer << endl;
      return;
    }
    char* ptr[3] = { buffer, NULL, NULL };
    char* end[3] = { NULL, NULL, buffer + n };
    end[1] = (char*)memchr(ptr[0], '=', end[2] - ptr[0]);
    ptr[2] = end[1] + 1;
    if (end[1][-1] == ')') end[1]--;
    end[0] = end[1];
    ptr[1] = (char*)memchr(ptr[0], '(', end[1] - ptr[0]);
    if (ptr[1]) end[0] = ptr[1]++; else ptr[1] = end[1];
    *end[0] = *end[1] = 0;
    Log_table.write(ptr[0], ptr[1], ptr[2], Global_thread_log_group);
  }
}
#else
struct _loghere {
  _loghere(bool state, int group = 0) {}
  operator bool() { return false; }
};
void _log() {}
void _cdecl _log(const char* format, ...) {}
#endif

const string& geom_name(int id) {
  static string names[mjMAXSZ];
  if (names[id].empty()) {
    const char* s;
    MJ((s = mj_id2name("geom", id)) ? mjCOM_OK : -1);
    if (*s) names[id] = s; else names[id] = string("geom ") + to_string(id);
  }
  return names[id];
}

// from http://www.euclideanspace.com/maths/geometry/rotations/conversions/quaternionToMatrix/
void quatToMatrix(float q[4], double m[3][3]) {
  double w = q[0], x = q[1], y = q[2], z = q[3];
  double sqw = w * w, sqx = x * x, sqy = y * y, sqz = z * z;
  double invs = 1 / (sqx + sqy + sqz + sqw);
  m[0][0] = (sqx - sqy - sqz + sqw) * invs;
  m[1][1] = (-sqx + sqy - sqz + sqw) * invs;
  m[2][2] = (-sqx - sqy + sqz + sqw) * invs;
  double tmp1 = x * y;
  double tmp2 = z * w;
  m[1][0] = 2.0 * (tmp1 + tmp2) * invs;
  m[0][1] = 2.0 * (tmp1 - tmp2) * invs;
  tmp1 = x * z;
  tmp2 = y * w;
  m[2][0] = 2.0 * (tmp1 - tmp2) * invs;
  m[0][2] = 2.0 * (tmp1 + tmp2) * invs;
  tmp1 = y * z;
  tmp2 = x * w;
  m[2][1] = 2.0 * (tmp1 + tmp2) * invs;
  m[1][2] = 2.0 * (tmp1 - tmp2) * invs;
}

double length(double a[3]) {
  return sqrt(sq(a[0]) + sq(a[1]) + sq(a[2]));
}

double dot(double a[3], double b[3]) {
  return a[0] * b[0] + a[1] * b[1] + a[2] * b[2];
}

void cross(double a[3], double b[3], double c[3]) {
  c[0] = a[1] * b[2] - a[2] * b[1];
  c[1] = a[2] * b[0] - a[0] * b[2];
  c[2] = a[0] * b[1] - a[1] * b[0];
}
