
struct NUT {

  mjInfo info;
  mjOneBody ob;
  string name;
  bool is_nut_or_bolt;
  double max_torque;
  double time_last_update;
  DWORD tickcount_last_message;
  int turn, last_turn_dir;
  int last_turn_countdown;
  double target_angvel;
  bool hold;
  double z_axis[3];
  double last_x_axis[3];
  double pos[3];
  double last_pos_length;
  double clampforce;
  double angvel;
  double torque;
  mjApplied applied;
  ofstream csv;
  bool has_reset;
  bool has_blocked;

  void update() {
    if (turn) last_turn_countdown = 3; else if (last_turn_countdown) last_turn_countdown--;
    _loghere lh(last_turn_countdown && !ps, 0);
    _log();

    MJ(mj_get_onebody(&ob));
    double dt = ob.time - time_last_update;
    if (dt < 0) {
      _log("\n************* Reset *************");
      has_reset = true;
      torque = 0;
    }
    if (dt < 1e-9) dt = 1e-9;
    time_last_update = ob.time;

    double m[3][3], x_axis[3];
    quatToMatrix(ob.quat, m);
    for_(d, 3) {
      x_axis[d] = m[d][0];
      z_axis[d] = m[d][2];
    }

    get_clampforce();
    if (clampforce >= 12000) {
      _log("\n************* Blocked *************");
      has_blocked = true;
    }
    _log("time(s)=%.4f", ob.time);
    //_log("dt(s)=%f", dt);
    //_log("z_axis(m)=%f %f %f", z_axis[0], z_axis[1], z_axis[2]);
    _log("pos(m)=%f %f %f", ob.pos[0], ob.pos[1], ob.pos[2]);

    double v = 0; for_(d, 3) v += sq(ob.pos[d] - pos[d]); assert(dt > 0); v = sqrt(v) / dt;
    _log("v(m/s)=%.9f", v);
    for_(d, 3) pos[d] = ob.pos[d];

    double dzang = acos(dot(last_x_axis, x_axis));
    double c[3]; cross(last_x_axis, x_axis, c);
    if (dot(c, z_axis) < 0) dzang = -dzang;
    //_log("dzang(mrad)=%d", round32(dzang * 1000));
    for_(d, 3) last_x_axis[d] = x_axis[d];

    double dzpos = dot(z_axis, pos) - last_pos_length;
    _log("dzpos(m)=%f", dzpos);
    last_pos_length = length(pos);

    _log("turn=%c ", (turn) ? (turn > 0) ? 'L' : 'R' : ' ');
    _log("cforce(mN)=%d", round32(clampforce * 1000));

    if (!turn) target_angvel = 0;
    else {
      const double map[] = {
        /* <=.001*/ 16000,
        /*    .01*/ 16000,
        /*     .1*/ 500,
        /*      1*/ 300,
        /*     10*/ 250,
        /*    100*/ 200,
        /*   1000*/ 150,
        /*>=10000*/ 50,
      };
      double i, f = modf(3 + log10(bound(.001, clampforce, 10000 - 1e-5)), &i);
      assert(i >= 0 && i + 1 < _countof(map));
      assert(f >= 0 && f < 1);
      target_angvel = (map[int(i)] * (1 - f) + map[int(i) + 1] * f) / 1000;
    }
    _log("target(angvel)=%d", round32(target_angvel * 1000));

    if (turn) last_turn_dir = turn;
    angvel = 0; for_(d, 3) angvel += last_turn_dir * ob.angvel[d] * z_axis[d];
    _log("angvel(mrad/s)=%d", bound(-999999, round32(angvel * 1000), 999999));

    double error = 1;
    if (target_angvel) error = bound(1 / 10.99, angvel / target_angvel, 10.99);
    _log("error(%%)=%d", round32((error < 1 ? 1 - 1 / error : error - 1) * 100));

    assert(dt > 0);
    if (!turn) torque = 0;
    else torque = bound(.0001, torque / pow(error, bound(.1, 0.4 * log10(dt) + 1.4, .5)), max_torque);
    _log("torque(mNm)=%d", round32(torque * 1000));

    for_(d, 3) applied.xfrc_applied[ob.bodyid][3 + d] = turn * torque * z_axis[d];
    mjtResult r = mj_set_applied(&applied);
    if (r != mjCOM_OK) {
      if (r == mjCOM_BADSIZE) {
        MJ(mj_get_applied(&applied));
        memset(&applied.xfrc_applied, 0, sizeof(applied.xfrc_applied));
        MJ(mj_set_applied(&applied));
      } else throw string("mj_set_applied failed\n\n") + hx_last_result();
    }

    if (capture_image_sequence) message(); else {
      DWORD now = GetTickCount();
      if (now - tickcount_last_message >= 50) {
        tickcount_last_message = now;
        message();
      }
    }

    if (csv.is_open() && !has_reset) {
      csv << ob.time;
      csv << ',' << lbound(1e-4, clampforce);
      csv << ',' << v;
      csv << ',' << angvel;
      csv << endl;
      verify(csv.good());
    }

    MJ(mj_step());
  }

  void open_csv(string filename) {
    if (csv.is_open()) csv.close();
    csv.open(filename);
    verify(csv.good());
    csv << "time,clampforce,v,angvel" << endl;
    csv << "s,N,m/s,rad/s" << endl;
    verify(csv.good());
  }

  void get_clampforce() {
    _loghere lh(turn && !ps, 1);
    mjContact c;
    MJ(mj_get_contact(&c));
    verify(c.ncon <= mjMAXSZ);
    double pos = 0, neg = 0, guess = 0;
    for_(i, c.ncon) {
      const char* name1 = geom_name(c.geom1[i]).c_str();
      const char* name2 = geom_name(c.geom2[i]).c_str();
      if (
        strstr(name1, "pressure")
        //!strstr(name1, "floor") && 
        //!strstr(name2, "cage base") &&
        //!strstr(name1, "cover fixation")
      ) {
        const double scale = 1;
        _log("%s(%s)=%d %d %d", name1, name2, round32(c.force[i][0] * scale), round32(c.force[i][1] * scale), round32(c.force[i][2] * scale));
      }
      int body1 = info.geom_bodyid[c.geom1[i]];
      int body2 = info.geom_bodyid[c.geom2[i]];
      int dir = (body1 == ob.bodyid) | -(body2 == ob.bodyid);
      if (dir) {
        //_log("dir=%d", dir);
        //_log("frame (0 1 2 )=%3d%3d%3d", round32(c.frame[i][0] * 99.499), round32(c.frame[i][1] * 99.499), round32(c.frame[i][2] * 99.499));
        //_log("frame (3 4 5 )=%3d%3d%3d", round32(c.frame[i][3] * 99.499), round32(c.frame[i][4] * 99.499), round32(c.frame[i][5] * 99.499));
        //_log("frame (6 7 8 )=%3d%3d%3d", round32(c.frame[i][6] * 99.499), round32(c.frame[i][7] * 99.499), round32(c.frame[i][8] * 99.499));
        //_log("z_axis=%3d%3d%3d", round32(z_axis[0] * 99.499), round32(z_axis[1] * 99.499), round32(z_axis[2] * 99.499));
        double af = 0; for_(d, 3) af += dir * c.frame[i][d] * z_axis[d] * c.force[i][0];
        //_log("af(mN)=%d", round32(af * 1000));
        if (af >= 0) pos += af; else neg += af;
        guess += c.force[i][0];
      }
    }
    guess /= 1 + 1 / cosd(30);
    //_log("pos=%d", round32(pos * 1000));
    //_log("neg=%d", round32(neg * 1000));
    //_log("guess=%d", round32(guess * 1000));
    clampforce = pos;
  }

  void message() {
    ostringstream ss;
    if (ps) ss << "Parameter Search" << endl << combination_name << endl << "loop " << loop << " ps " << ps;
    else ss << name;
    if (capture_image_sequence) ss << endl << "time: " << fixed << setprecision(4) << ob.time;
    if (is_nut_or_bolt) {
      if (ob.isfloating) {
        ss << fixed;
        if (!capture_image_sequence) {
          ss << endl << "torque: " << setprecision(3) << (turn > 0 ? "-" : "") << torque << " Nm";
          ss << endl << "   max: " << setprecision(1) << max_torque << " [PgUp]";
        }
        ss << endl << "cforce: " << setprecision(0) << clampforce << " N";
      } else ss << endl << "Not a floating body" << endl << "Add a free joint";
    } else ss << endl << "This model contains" << endl << "no nuts or bolts";
    MJ(mj_message(ss.str().c_str()));
  }

  void find(int dir) {
    MJ(mj_get_applied(&applied));
    memset(&applied.xfrc_applied, 0, sizeof(applied.xfrc_applied));
    torque = 0;
    max_torque = 9.5;
    is_nut_or_bolt = false;
    for (int n = info.nbody + 1; n && !is_nut_or_bolt; n--) {
      ob.bodyid = (ob.bodyid + dir + info.nbody) % info.nbody;
      const char* s;
      MJ((s = mj_id2name("body", ob.bodyid)) ? mjCOM_OK : -1);
      if (*s) {
        name = s;
        static const regex re("\\bnut|bolt\\b");
        is_nut_or_bolt = regex_search(name, re) && name.find("weld nut") == string::npos;
        if (is_nut_or_bolt) {
          static regex re("\\bmax ([-+]?[0-9]*\\.?[0-9]+([eE][-+]?[0-9]+)?) ?Nm\\b");
          smatch m;
          if (regex_search(name, m, re)) max_torque = stod(m[1]);
        }
      } else {
        name = string("body ") + to_string(ob.bodyid);
        is_nut_or_bolt = false;
      }
    }
  }

  void copy_position_to_clipboard() {
    string s = string("pos=\"") + to_string(ob.pos[0]) + ' ' + to_string(ob.pos[1]) + ' ' + to_string(ob.pos[2]) + '"';
    s += " quat=\"" + to_string(ob.quat[0]) + ' ' + to_string(ob.quat[1]) + ' ' + to_string(ob.quat[2]) + ' ' + to_string(ob.quat[3]) + '"';
    SetClipboardText(s.c_str());
  }

  void wait_until_time_resets() {
    MJ(mj_step());
    MJ(mj_get_onebody(&ob));
    float time_zero = ob.time;
    do {
      MJ(mj_step());
      MJ(mj_get_onebody(&ob));
    } while (ob.time == time_zero);
    time_last_update = ob.time - .0005;
    torque = 0;
    MJ(mj_get_applied(&applied));
    memset(&applied.xfrc_applied, 0, sizeof(applied.xfrc_applied));
    MJ(mj_set_applied(&applied));
  }

};
