
struct PARAMETER_SEARCH {

  vector<string> names;
  vector<vector<string>> options;
  vector<string> combinations;
  string current_combination;
  string xml_template;

  void init_options() {
    struct {
      const char* name;
      const char* options;
    } data[] = {
      "contact_friction_sliding", ".75 .8 .85 .9 .95",
      "contact_solimp_dmin", ".994 .996 .997 .998 .999",
      "contact_solimp_width", ".00005 .00008 .0001 .00012 .00015",
      "contact_solref_timeconst", ".0001 .0002 .0003",
      "contact_solref_dampratio", ".6 .8 1 1.2 1.4",
      "spacer_friction_sliding", "1.1 1.3 1.5 1.7 2",
      "spacer_solimp_dmin", ".988 .990 .992 .994 .996",
      "spacer_solimp_width", "0 .00007 .00008 .00009 .00010",
      "spacer_solref_timeconst", ".00006 .00008 .00010 .00012 .00014",
      "spacer_solref_dampratio", ".6 .8 1 1.2",
    };
    for (auto& d : data) {
      names.push_back(d.name);
      istringstream ss(d.options);
      vector<string> os;
      while (!ss.eof()) {
        string o;
        getline(ss, o, ' ');
        os.push_back(o);
      }
      options.push_back(os);
    }
  }

  void read_xml_template() {
    ifstream fs("Parameter Search\\Template.xml");
    verify(fs.good());
    stringstream ss;
    ss << fs.rdbuf();
    xml_template = ss.str();
  }

  void read_combinations() {
    ifstream fs("Parameter Search\\Combinations.txt");
    if (fs.good()) {
      string c;
      while (getline(fs, c)) combinations.push_back(c);
      verify(fs.eof());
    }
  }

  void new_random_combination() {
  try_another:
    current_combination.clear();
    for (auto& o : options) {
      if (!current_combination.empty()) current_combination += ',';
      current_combination += o[rand() * o.size() / (RAND_MAX + 1)];
    }
    for (auto& known : combinations) {
      if (current_combination == known) goto try_another;
    }
  }

  void write_current_combination_xml() {
    istringstream ss(current_combination);
    string xml = regex_replace(xml_template, regex("\\{current_combination\\}"), combination_name);
    for (auto& n : names) {
      regex re(string("\\{") + n + "\\}");
      string o;
      getline(ss, o, ',');
      xml = regex_replace(xml, re, o);
    }
    ofstream fs("Parameter Search\\current combination.xml");
    verify(fs.good());
    fs << xml;
    verify(fs.good());
  }

  void append_current_combination() {
    ofstream fs("Parameter Search\\combinations.txt", ios::app);
    verify(fs.good());
    fs << current_combination << endl;
    combinations.push_back(current_combination);
  }

  void reload_model() {
    while (GetForegroundWindow() != Haptix_window) Sleep(200);
    char title[256];
    verify(GetWindowTextA(Haptix_window, title, sizeof(title)));
    if (strstr(title, "MuJoCo HAPTIX : Parameter Search\\combination ")) {
      Send(CTRL + 'L');
    } else {
      PostMessage(Haptix_window, WM_COMMAND, 1100, 0);
      verify(WinWaitActive("Open model", 1000));
      SetClipboardTextW(GetProgramDirectory("Parameter Search\\current combination.xml"));
      Send(CTRL + 'V');
      Send(VK_RETURN);
      verify(WinWaitActive("MuJoCo HAPTIX : Parameter Search\\combination ", 3000));
    }
  }

  double target_time;
  double target_clampforce;
  double avg_clampforce;

  void on_off() {
    ps = !ps;
    if (ps) {
      if (options.empty()) init_options();
      if (xml_template.empty()) read_xml_template();
      if (combinations.empty()) read_combinations();
      srand(time(NULL));
    }
  }

  
  float washer_pos[3];
  double avg_torque;

  int update() {
    avg_clampforce = avg_clampforce * .7 + nut.clampforce * .3;
    avg_torque = avg_torque * .7 + nut.torque * .3;
    mjOneBody obWasher = {nut.ob.bodyid - 1};
    if (ps != 1) {
      MJ(mj_get_onebody(&obWasher));
      double dpos = 0;
      for_(d, 3) dpos += sq(obWasher.pos[d] - washer_pos[d]);
      dpos = sqrt(dpos);
      if (dpos > .0003 || nut.ob.time > 300 || nut.has_reset || nut.has_blocked) {
        append_current_combination();
        ps = 1;
      }
    }
    switch (ps) {
    default:
      throw string("Undefined statement ps=") + to_string(ps) + " in parameter_search::update()";
      // try a new combination
    case 1:
      combination_name = string("combination ") + to_string(combinations.size() + 1);
      new_random_combination();
      write_current_combination_xml();
      reload_model();
      nut.wait_until_time_resets();
      nut.open_csv("Parameter Search\\current combination.tmp");
      nut.has_reset = nut.has_blocked = false;
      avg_clampforce = avg_torque = 0;
      MJ(mj_get_onebody(&obWasher));
      for_(d, 3) washer_pos[d] = obWasher.pos[d];
      loop = 0;
      // tighten until clampforce > 6000
    case 2:
      nut.turn = -1;
      if (avg_clampforce < 6000) return 2;
      // tighten until torque > 9
    case 3:
      nut.turn = -1;
      if (avg_torque < 9) return 3;
      target_time = nut.ob.time + .5;
      // tighten for another .5 s
    case 4: 
      nut.turn = -1;
      if (nut.ob.time < target_time) return 4;
      // wait for .5 s
      target_time = nut.ob.time + .5;
    case 5:
      if (nut.ob.time < target_time) return 5;
      // loosen until clampforce < 1
    case 6: 
      nut.turn = +1;
      if (avg_clampforce > 1) return 6;
      target_time = nut.ob.time + .2;
      // loosen for another .2 s
    case 7: 
      nut.turn = +1;
      if (nut.ob.time < target_time) return 7;
      // wait for .2 s
      target_time = nut.ob.time + .2;
    case 8:
      if (nut.ob.time < target_time) return 8;
      if (++loop < 3) return 2;
      // good combination found
      nut.csv.close();
      verify(MoveFileExA("Parameter Search\\current combination.tmp", ("Parameter Search\\" + combination_name + ".csv").c_str(), MOVEFILE_REPLACE_EXISTING));
      verify(MoveFileExA("Parameter Search\\current combination.xml", ("Parameter Search\\" + combination_name + ".xml").c_str(), MOVEFILE_REPLACE_EXISTING));
      append_current_combination();
      return 1;
    }
  }

};
